// --------------------------------------------------------------------------------------------------------------------
// <copyright file="DefaultRegistry.cs" company="Web Advanced">
// Copyright 2012 Web Advanced (www.webadvanced.com)
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace $safeprojectname$.DependencyResolution
{
	using Entities;
	using FluentValidation;
	using FluentValidation.Mvc;
	using Infrastructure;
	using Infrastructure.Validation;
	using Mehdime.Entity;
	using StructureMap.Configuration.DSL;
	using StructureMap.Graph;
	using StructureMap.Pipeline;
	using System.Web.Mvc;

	public class DefaultRegistry : Registry
	{
		#region Constructors and Destructors

		public DefaultRegistry()
		{
			Scan(
							aAssemblyScanner =>
							{
								aAssemblyScanner.TheCallingAssembly();
								aAssemblyScanner.WithDefaultConventions();
								aAssemblyScanner.LookForRegistries();
								aAssemblyScanner.AssemblyContainingType<DefaultRegistry>();
								aAssemblyScanner.AddAllTypesOf(typeof(IModelBinder));
								aAssemblyScanner.AddAllTypesOf(typeof(IModelBinderProvider));
								aAssemblyScanner.With(new ControllerConvention());
							});
			//For<Model>().Use(() => new Model()).LifecycleIs<TransientLifecycle>();
			For<IControllerFactory>().Use<ControllerFactory>();
			For<ModelValidatorProvider>().Use<FluentValidationModelValidatorProvider>();
			For<IDbContextFactory>().Use<DbContextFactory>().LifecycleIs<ContainerLifecycle>();
			For<IDbContextScopeFactory>().Use<DbContextScopeFactory>().LifecycleIs<ContainerLifecycle>();
			For<IAmbientDbContextLocator>().Use<AmbientDbContextLocator>().LifecycleIs<ContainerLifecycle>();
			For<IValidatorFactory>().Use<StructureMapValidatorFactory>();
		}

		#endregion
	}
}
