// --------------------------------------------------------------------------------------------------------------------
// <copyright file="StructureMapDependencyScope.cs" company="Web Advanced">
// Copyright 2012 Web Advanced (www.webadvanced.com)
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace $safeprojectname$.DependencyResolution
{
	using Microsoft.Practices.ServiceLocation;
	using StructureMap;
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using System.Web;

	/// <summary>
	/// The structure map dependency scope.
	/// </summary>
	public class StructureMapDependencyScope : ServiceLocatorImplBase
	{
		#region Constants and Fields

		private const string NestedContainerKey = "Nested.Container.Key";

		#endregion

		#region Constructors and Destructors

		public StructureMapDependencyScope(IContainer aContainer)
		{
			if (aContainer == null)
			{
				throw new ArgumentNullException(paramName: "container");
			}
			Container = aContainer;
		}

		#endregion

		#region Public Properties

		public IContainer Container { get; set; }

		public IContainer CurrentNestedContainer
		{
			get
			{
				return (IContainer)HttpContext.Items[NestedContainerKey];
			}
			set
			{
				HttpContext.Items[NestedContainerKey] = value;
			}
		}

		#endregion

		#region Properties

		private HttpContextBase HttpContext
		{
			get
			{
				HttpContextBase context = Container.TryGetInstance<HttpContextBase>();
				return context ?? new HttpContextWrapper(System.Web.HttpContext.Current);
			}
		}

		#endregion

		#region Public Methods and Operators

		public void CreateNestedContainer()
		{
			if (CurrentNestedContainer != null)
			{
				return;
			}
			CurrentNestedContainer = Container.GetNestedContainer();
		}

		public void Dispose()
		{
			DisposeNestedContainer();
			Container.Dispose();
		}

		public void DisposeNestedContainer()
		{
			if (CurrentNestedContainer != null)
			{
				CurrentNestedContainer.Dispose();
				CurrentNestedContainer = null;
			}
		}

		public IEnumerable<object> GetServices(Type aServiceType)
		{
			return DoGetAllInstances(aServiceType);
		}

		#endregion

		#region Methods

		protected override IEnumerable<object> DoGetAllInstances(Type aServiceType)
		{
			return (CurrentNestedContainer ?? Container).GetAllInstances(aServiceType).Cast<object>();
		}

		protected override object DoGetInstance(Type aServiceType, string aKey)
		{
			IContainer container = (CurrentNestedContainer ?? Container);

			if (string.IsNullOrEmpty(aKey))
			{
				return aServiceType.IsAbstract || aServiceType.IsInterface
						? container.TryGetInstance(aServiceType)
						: container.GetInstance(aServiceType);
			}

			return container.GetInstance(aServiceType, aKey);
		}

		#endregion
	}
}
