﻿namespace $safeprojectname$.Features.User
{
	using AutoMapper;
	using Entities;
	using FluentValidation;
	using Infrastructure.MediatedControllers;
	using Infrastructure.RequestHandlers;
	using MediatR;
	using Mehdime.Entity;
	using System.ComponentModel.DataAnnotations;

	public class Create
	{
		public class Query : IRequest<Command> { }

		public class QueryHandler : IRequestHandler<Query, Command>
		{
			public Command Handle(Query aQuery)
			{
				return new Command();
			}
		}

		public class Command : IRequest
		{
			public string LastName { get; set; }

			[Display(Name = "First Name")]
			public string FirstAndMiddleName { get; set; }
		}

		public class CommandValidator : AbstractValidator<Command>
		{
			public CommandValidator()
			{
				RuleFor(aCommand => aCommand.LastName).NotNull().Length(min: 1, max: 50);
				RuleFor(aCommand => aCommand.FirstAndMiddleName).NotNull().Length(min: 1, max: 50);
			}
		}

		public class CommandHandler : DbContextCommandHandler<Command, Model>
		{
			public CommandHandler(IDbContextScopeFactory aDbContextScopeFactory) : base(aDbContextScopeFactory) { }

			protected override void HandleInScope(Command aCommand, Model aModel)
			{
				User user = Mapper.Map<Command, User>(aCommand);
				aModel.Users.Add(user);
			}
		}

		public class UiController : MediatedGetPostController<Query, Command, Index.UiController>
		{
			public UiController(IMediator aMediator) : base(aMediator) { }
		}

		public class MappingProfile : Profile
		{
			protected override void Configure()
			{
				CreateMap<Command, User>();
			}
		}
	}
}