﻿namespace $safeprojectname$.Features.User
{
	using AutoMapper;
	using Entities;
	using MediatR;
	using Mehdime.Entity;
	using System.ComponentModel.DataAnnotations;
	using System.Linq;
	using System.Threading.Tasks;
	using Infrastructure.MediatedControllers;
	using Infrastructure.RequestHandlers;

	public class Delete
	{
		public class Query : IAsyncRequest<Command>
		{
			public int UserId { get; set; }
		}

		public class QueryHandler : AsyncDbContextQueryHandler<Query, Command, Model>
		{
			public QueryHandler(IDbContextScopeFactory aDbContextScopeFactory) : base(aDbContextScopeFactory) { }

			protected override async Task<Command> HandleInScope(Query aQuery, Model aModel)
			{
				return await aModel.Users.Where(aUser => aUser.UserId == aQuery.UserId).ProjectToSingleOrDefaultAsync<Command>();
			}
		}
		public class Command : IAsyncRequest
		{
			public int UserId { get; set; }
			[Display(Name = "First Name")]
			public string FirstAndMiddleName { get; set; }
			public string LastName { get; set; }
		}

		public class ComandHandler : AsyncDbContextCommandHandler<Command, Model>
		{
			public ComandHandler(IDbContextScopeFactory aDbContextScopeFactory) : base(aDbContextScopeFactory) { }

			protected override async Task HandleInScope(Command aCommand, Model aModel)
			{
				User user = await aModel.Users.FindAsync(aCommand.UserId);
				aModel.Users.Remove(user);
			}
		}

		public class UiController : MediatedAsyncGetPostController<Query, Command, Index.UiController>
		{
			public UiController(IMediator aMediator) : base(aMediator) { }
		}

		public class MappingProfile : Profile
		{
			protected override void Configure()
			{
				CreateMap<User, Command>().ReverseMap();
			}
		}
	}
}