﻿namespace $safeprojectname$.Features.User
{
	using AutoMapper;
	using Entities;
	using Infrastructure.Mapping;
	using Infrastructure.MediatedControllers;
	using Infrastructure.RequestHandlers;
	using MediatR;
	using Mehdime.Entity;
	using PagedList;
	using System;
	using System.ComponentModel.DataAnnotations;
	using System.Linq;

	public class Index
	{

		public class Query : IRequest<Result>
		{
			public string SortOrder { get; set; }
			public string CurrentFilter { get; set; }
			public string SearchString { get; set; }
			public int? Page { get; set; }
		}

		public class QueryHandler : DbContextQueryHandler<Query, Result, Model>
		{
			public QueryHandler(IDbContextScopeFactory aDbContextScopeFactory) : base(aDbContextScopeFactory) { }

			protected override Result HandleInScope(Query aQuery, Model aModel)
			{
				var result = new Result
				{
					CurrentSort = aQuery.SortOrder,
					NameSortParm = String.IsNullOrEmpty(aQuery.SortOrder) ? "name_desc" : "",
				};

				if (aQuery.SearchString != null)
				{
					aQuery.Page = 1;
				}
				else
				{
					aQuery.SearchString = aQuery.CurrentFilter;
				}

				result.CurrentFilter = aQuery.SearchString;
				result.SearchString = aQuery.SearchString;

				IQueryable<User> users = from s in aModel.Users select s;
				if (!String.IsNullOrEmpty(aQuery.SearchString))
				{
					users = users.Where(aUser => aUser.LastName.Contains(aQuery.SearchString) || aUser.FirstAndMiddleName.Contains(aQuery.SearchString));
				}
				switch (aQuery.SortOrder)
				{
					case "name_desc":
						users = users.OrderByDescending(aUser => aUser.LastName);
						break;
					default: // Name ascending 
						users = users.OrderBy(aUser => aUser.LastName);
						break;
				}

				int pageSize = 3;
				int pageNumber = (aQuery.Page ?? 1);
				result.Users = users.ProjectToPagedList<Result.User>(pageNumber, pageSize);

				return result;
			}
		}



		public class Result : IRequest
		{
			public string CurrentSort { get; set; }
			public string NameSortParm { get; set; }
			public string CurrentFilter { get; set; }
			public string SearchString { get; set; }
			public IPagedList<User> Users { get; set; }

			public class User
			{
				public int UserId { get; set; }
				[Display(Name = "First Name")]
				public string FirstAndMiddleName { get; set; }
				public string LastName { get; set; }
			}
		}

		public class UiController : MediatedGetController<Query, Result>
		{
			public UiController(IMediator aMediator) : base(aMediator) { }
		}

		public class MappingProfile : Profile
		{
			protected override void Configure()
			{
				CreateMap<User, Index.Result.User>();
			}
		}
	}
}