﻿namespace $safeprojectname$.Infrastructure.Extensions
{
	using App_Start;
	using HtmlTags;
	using HtmlTags.Conventions;
	using HtmlTags.Conventions.Elements;
	using System;
	using System.Collections.Generic;
	using System.IO;
	using System.Linq.Expressions;
	using System.Web;
	using System.Web.Mvc;
	using System.Web.Mvc.Html;
	using System.Web.Routing;

	public static class HtmlHelperExtensions
	{
		public static string RequireJs(this HtmlHelper aHtmlHelper)
		{
			RouteValueDictionary values = aHtmlHelper.ViewContext.RouteData.Values;
			string controllerName = values["controller"].ToString();
			var viewContext = (RazorView)aHtmlHelper.ViewContext.View;
			string viewName = Path.GetFileNameWithoutExtension(viewContext.ViewPath);
			string requirePath = VirtualPathUtility.ToAbsolute(virtualPath: "~/Content/js/lib/require-2.1.18.js");

			string loc = string.Format(format: "~/Features/{0}/{1}.js", arg0: controllerName, arg1: viewName);

			if (File.Exists(aHtmlHelper.ViewContext.HttpContext.Server.MapPath(loc)))
			{
				string scriptBlock = "<script data-main='{0}' src='{1}'></script>";

				return string.Format(scriptBlock, VirtualPathUtility.ToAbsolute(loc), requirePath);
			}

			return string.Empty;
		}

		public static HtmlTag Input<T>(this HtmlHelper<T> aHtmlHelper, Expression<Func<T, object>> aExpression)
				where T : class
		{
			IElementGenerator<T> generator = GetGenerator(aHtmlHelper.ViewData.Model);
			return generator.InputFor(aExpression);
		}

		public static HtmlTag Label<T>(this HtmlHelper<T> aHtmlHelper, Expression<Func<T, object>> aExpression)
				where T : class
		{
			IElementGenerator<T> generator = GetGenerator(aHtmlHelper.ViewData.Model);
			return generator.LabelFor(aExpression);
		}

		public static HtmlTag Display<T>(this HtmlHelper<T> aHtmlHelper, Expression<Func<T, object>> aExpression)
				where T : class
		{
			IElementGenerator<T> generator = GetGenerator(aHtmlHelper.ViewData.Model);
			return generator.DisplayFor(aExpression);
		}

		public static HtmlTag DisplayLabel<T>(this HtmlHelper<T> aHtmlHelper, Expression<Func<T, object>> aExpression)
				where T : class
		{
			IElementGenerator<T> generator = GetGenerator(aHtmlHelper.ViewData.Model);
			return generator.TagFor(aExpression, category: "DisplayLabels");
		}

		public static HtmlTag DisplayLabel<T>(this HtmlHelper<IList<T>> aHtmlHelper, Expression<Func<T, object>> aExpression)
				where T : class
		{
			IElementGenerator<T> generator = GetGenerator(default(T));
			return generator.TagFor(aExpression, category: "DisplayLabels");
		}

		public static MvcHtmlString DisplayNameFor<TModel, TValue>(this HtmlHelper<IList<TModel>> aHtmlHelper,
				Expression<Func<TModel, TValue>> aExpression)
		{
			return new HtmlHelper<IEnumerable<TModel>>(aHtmlHelper.ViewContext, aHtmlHelper.ViewDataContainer).DisplayNameFor(aExpression);
		}

		public static HtmlTag InputBlock<T>(this HtmlHelper<T> aHtmlHelper,
				Expression<Func<T, object>> aExpression,
				Action<HtmlTag> aInputModifier = null) where T : class
		{
			aInputModifier = aInputModifier ?? (aHtmlTag => { });

			var divTag = new HtmlTag(tag: "div");
			divTag.AddClass(className: "col-md-10");

			HtmlTag inputTag = aHtmlHelper.Input(aExpression);
			aInputModifier(inputTag);

			divTag.Append(inputTag);

			return divTag;
		}

		public static HtmlTag FormBlock<T>(this HtmlHelper<T> aHtmlHelper,
				Expression<Func<T, object>> aExpression,
				Action<HtmlTag> aLabelModifier = null,
				Action<HtmlTag> aInputBlockModifier = null,
				Action<HtmlTag> aInputModifier = null
				) where T : class
		{
			aLabelModifier = aLabelModifier ?? (aHtmlTag => { });
			aInputBlockModifier = aInputBlockModifier ?? (aHtmlTag => { });

			var divTag = new HtmlTag(tag: "div");
			divTag.AddClass(className: "form-group");

			HtmlTag labelTag = aHtmlHelper.Label(aExpression);
			aLabelModifier(labelTag);

			HtmlTag inputBlockTag = aHtmlHelper.InputBlock(
					aExpression,
					aInputModifier);
			aInputBlockModifier(inputBlockTag);

			divTag.Append(labelTag);
			divTag.Append(inputBlockTag);

			return divTag;
		}

		public static HtmlTag ValidationDiv(this HtmlHelper aHtmlHelper)
		{
			return new HtmlTag(tag: "div")
					.Id(id: "validationSummary")
					.AddClass(className: "alert")
					.AddClass(className: "alert-danger")
					.AddClass(className: "hidden");
		}

		public static HtmlTag FeatureLink<TController>(this HtmlHelper aHtmlHelper, string aLinkText) where TController : Controller
		{
			string controllerName, actionName;

			LinkBuilder.GetNamesFromController<TController>(out controllerName, out actionName);
			string url = $"{controllerName}/{actionName}";

			return Link(aHtmlHelper, aLinkText, url);
		}


		public static HtmlTag FeatureLink<TController>(this HtmlHelper aHtmlHelper, string aLinkText, object aRouteValues) where TController : Controller
		{
			string controllerName, actionName;
			LinkBuilder.GetNamesFromController<TController>(out controllerName, out actionName);

			var routeValueDictionary = new RouteValueDictionary(aRouteValues);
			routeValueDictionary.Add(key: "Controller", value: controllerName);
			routeValueDictionary.Add(key: "Action", value: actionName);
			var urlHelper = new UrlHelper(aHtmlHelper.ViewContext.RequestContext);
			string url = urlHelper.Action(actionName, controllerName, routeValueDictionary);

			return Link(aHtmlHelper, aLinkText, url);
		}

		private static HtmlTag Link(HtmlHelper aHtmlHelper, string aLinkText, string aUrl)
		{
			aUrl = "~/" + aUrl;
			aUrl = UrlHelper.GenerateContentUrl(aUrl, aHtmlHelper.ViewContext.HttpContext);

			return new HtmlTag(tag: "a", configure: aHtmlTag =>
			{
				aHtmlTag.Text(aLinkText);
				aHtmlTag.Attr(attribute: "href", value: aUrl);
			});
		}

		private static IElementGenerator<T> GetGenerator<T>(T aModel) where T : class
		{
			HtmlConventionLibrary library = StructuremapMvc.StructureMapDependencyScope.CurrentNestedContainer.GetInstance<HtmlConventionLibrary>();
			return ElementGenerator<T>.For(library, aType => StructuremapMvc.StructureMapDependencyScope.CurrentNestedContainer.GetInstance(aType), aModel);
		}
	}
}