﻿namespace $safeprojectname$.Infrastructure.Tags
{
	using System;
	using System.ComponentModel.DataAnnotations;
	using System.Drawing;
	using System.Web.Mvc;
	using HtmlTags;
	using HtmlTags.Conventions;
	using HtmlTags.Conventions.Elements;
	using HtmlTags.Reflection;

	public class DefaultAspNetMvcHtmlConventions : HtmlConventionRegistry
	{
		public DefaultAspNetMvcHtmlConventions()
		{
			Editors.Always.AddClass(className: "form-control");

			//Label
			Labels.Always.AddClass(className: "control-label");
			Labels.Always.AddClass(className: "col-md-2");
			Labels.ModifyForAttribute<DisplayAttribute>((aHtmlTag, a) => aHtmlTag.Text(a.Name));

			//Checkbox
			Editors.IfPropertyIs<bool>().Attr(attName: "type", value: "checkbox");

			//Color
			Editors.IfPropertyIs<Color>().Attr(attName: "type", value: "color");

			//Date/Time/DateTime/Local DateTime
			Editors.IfPropertyIs<DateTime?>().ModifyWith(aElementRequest => aElementRequest.CurrentTag
					.AddPattern(aPattern: "9{1,2}/9{1,2}/9999")
					.AddPlaceholder(aPlaceholder: "MM/DD/YYYY")
					.AddClass(className: "datepicker")
				.Value(aElementRequest.Value<DateTime?>() != null ? aElementRequest.Value<DateTime>().ToShortDateString() : string.Empty));
			Displays.IfPropertyIs<DateTime>().ModifyWith(aElementRequest => aElementRequest.CurrentTag.Text(aElementRequest.Value<DateTime>().ToShortDateString()));
			Displays.IfPropertyIs<DateTime?>().ModifyWith(aElementRequest => aElementRequest.CurrentTag.Text(aElementRequest.Value<DateTime?>() == null ? null : aElementRequest.Value<DateTime?>().Value.ToShortDateString()));


			//Email
			Editors.If(aElementRequest => aElementRequest.Accessor.Name.Contains(value: "Email"))
					.Attr(attName: "type", value: "email");

			//Hidden
			Editors.IfPropertyIs<Guid>().Attr(attName: "type", value: "hidden");
			Editors.IfPropertyIs<Guid?>().Attr(attName: "type", value: "hidden");
			Editors.IfPropertyHasAttribute<HiddenInputAttribute>().Attr(attName: "type", value: "hidden");

			//Number
			Editors.IfPropertyIs<decimal?>().ModifyWith(aElementRequest => aElementRequest.CurrentTag
					.Data(key: "pattern", value: "9{1,9}.99")
					.Data(key: "placeholder", value: "0.00"));
			//TODO int float
			Displays.IfPropertyIs<decimal>().ModifyWith(aElementRequest => aElementRequest.CurrentTag.Text(aElementRequest.Value<decimal>().ToString(format: "C")));

			//Password
			Editors.If(aElementRequest => aElementRequest.Accessor.Name.Contains(value: "Password"))
					.Attr(attName: "type", value: "password");
			Editors.If(aElementRequest =>
			{
				DataTypeAttribute dataTypeAttribute = aElementRequest.Accessor.GetAttribute<DataTypeAttribute>();
				return dataTypeAttribute != null && dataTypeAttribute.DataType == DataType.Password;
			}).Attr(attName: "type", value: "password");

			//Radio
			Editors.Modifier<EnumDropDownModifier>();

			//Telephone
			Editors.If(aElementRequest => aElementRequest.Accessor.Name.Contains(value: "Phone"))
					.Attr(attName: "type", value: "tel");
			Editors.If(aElementRequest =>
			{
				DataTypeAttribute dataTypeAttribute = aElementRequest.Accessor.GetAttribute<DataTypeAttribute>();
				return dataTypeAttribute != null && dataTypeAttribute.DataType == DataType.PhoneNumber;
			}).Attr(attName: "type", value: "tel");

			//Text
			// Default

			//Url
			Editors.If(aElementRequest => aElementRequest.Accessor.Name.Contains(value: "Url"))
					.Attr(attName: "type", value: "url");
			Editors.If(aElementRequest =>
			{
				DataTypeAttribute dataTypeAttribute = aElementRequest.Accessor.GetAttribute<DataTypeAttribute>();
				return dataTypeAttribute != null && dataTypeAttribute.DataType == DataType.Url;
			}).Attr(attName: "type", value: "url");

			//Id
			Editors.If(aElementRequest => aElementRequest.Accessor.Name.EndsWith(value: "id", comparisonType: StringComparison.OrdinalIgnoreCase)).BuildBy(a => new HiddenTag().Value(a.StringValue()));

			//byte
			Editors.IfPropertyIs<byte[]>().BuildBy(a => new HiddenTag().Value(Convert.ToBase64String(a.Value<byte[]>())));

			//Domain Specific

			//Editors.BuilderPolicy<UserSelectElementBuilder>();
			//Editors.BuilderPolicy<XXXSelectElementBuilder>();
			DisplayLabels.Always.BuildBy<DefaultDisplayLabelBuilder>();

			DisplayLabels.ModifyForAttribute<DisplayAttribute>((aHtmlTag, a) => aHtmlTag.Text(a.Name));

		}

		public ElementCategoryExpression DisplayLabels
		{
			get { return new ElementCategoryExpression(Library.TagLibrary.Category(category: "DisplayLabels").Profile(TagConstants.Default)); }
		}
	}
}